/* bzflag
 * Copyright (c) 1993 - 2003 Tim Riker
 *
 * This package is free software;  you can redistribute it and/or
 * modify it under the terms of the license found in the file
 * named LICENSE that should have accompanied this file.
 *
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */
/* SFile.cpp : Defines the entry point for the DLL application.
* 01/16/03: Inital implementaion: Jeff Myers
*/

#include "OSFile.h"

#ifdef WIN32
  #define strcasecmp stricmp
  #pragma warning(disable : 4786)  // Disable warning message
#else
 /* char *strupr(char *s)
  {
    char *ptr = s;
    while(*ptr) {
      *ptr = toupper(*ptr);
      ptr++;
    }
    return s;
  }*/
#endif

typedef std::vector<std::string> FileNameList;

#ifdef WIN32
  #define strcasecmp stricmp
  #pragma warning(disable : 4786)  // Disable warning message
#else
  char *strupr(char *s)
  {
    char *ptr = s;
    while(*ptr) {
      *ptr = toupper(*ptr);
      ptr++;
    }
    return s;
  }
#endif

// these are the core file name conversions
// std pathnames are unix style
// if you are adding a platform
// this is one place you need to add conversions to/from your OS

// util functions
void OSFileStdToOSDir (char *szDir)
{
#ifndef WIN32
	return;
#else
	char *pchar = szDir;

	while (*pchar != NULL) {
		if (*pchar == '/')
			*pchar = '\\';

		pchar++;
	}
#endif//WIN32
}

void OSFileOSToStdDir (char *szDir)
{
#ifndef WIN32
	return;
#else
	char *pchar = szDir;

	while (*pchar != NULL) {
		if (*pchar == '\\')
			*pchar = '/';

		pchar++;
	}
#endif//WIN32
}


OSFile::OSFile ()
{
	pFP = NULL;
	sStdName[0] = NULL;
	sOSName[0] = NULL;
}

OSFile::OSFile (const char *name)
{
	pFP = NULL;
	sStdName[0] = NULL;
	sOSName[0] = NULL;
	stdName(name);
}

OSFile::OSFile (const char *name, bool bRead, bool bText)
{
	pFP = NULL;
	sStdName[0] = NULL;
	sOSName[0] = NULL;
	open(name,bRead,bText);
}

OSFile::~OSFile()
{
	close();
}

bool OSFile::open(const char *name, bool bRead, bool bText)
{
	close();

	stdName(name);

	return open(bRead,bText);
}

bool OSFile::open (bool bRead, bool bText)
{
	std::string	mode;

	if (bRead)
		mode = "r";
	else
		mode = "w";
		
	if (bText)
		mode += "t";
	else
		mode += "b";

	pFP = fopen(sOSName,mode.c_str());

	return isOpen();
}

bool OSFile::close (void)
{
	if (pFP)
		fclose(pFP);

	pFP = NULL;

	return (!isOpen());
}

void OSFile::stdName (const char *name)
{
	strcpy(sStdName,name);
	strcpy(sOSName,name);
	OSFileStdToOSDir(sOSName);
}

void OSFile::osName (const char *name)
{
	strcpy(sStdName,name);
	strcpy(sOSName,name);
	OSFileOSToStdDir(sStdName);
}

const char* OSFile::getExtension (void)
{
	if (strlen(sStdName)== 0)
		return NULL;

	char	*pEnd = strrchr(sStdName,'.');

	if (pEnd)
		pEnd++;

	return (const char*)pEnd;
}

FILE* OSFile::getFile (void)
{
	return pFP;
}

const char* OSFile::getStdName (void)
{
	return sStdName;
}

const char* OSFile::getOSName (void)
{
	return sOSName;
}

bool OSFile::isOpen (void)
{
	return pFP != NULL;
}

// OS Dir classes

OSDir::OSDir()
{
	sBaseStdDir[0] = NULL;
	sBaseOSDir[0] = NULL;
	iNamePos = -1;
}

OSDir::OSDir(const char* dirName)
{
	sBaseStdDir[0] = NULL;
	sBaseOSDir[0] = NULL;
	setStdDir(dirName);
	iNamePos = -1;

	pNameList = (void*) new FileNameList;
}

OSDir::~OSDir()
{
	if (pNameList)
		((FileNameList*)pNameList)->clear();

	delete((FileNameList*)pNameList);
	pNameList = NULL;
}

void OSDir::setStdDir (const char* dirName)
{
	strcpy(sBaseStdDir,dirName);
	strcpy(sBaseOSDir,dirName);
	OSFileStdToOSDir(sBaseOSDir);
}

void OSDir::setOSDir (const char* dirName)
{
	strcpy(sBaseStdDir,dirName);
	strcpy(sBaseOSDir,dirName);
	OSFileOSToStdDir(sBaseStdDir);
}

void OSDir::makeStdDir (const char* dirName)
{
	setStdDir(dirName);
	//	mkdir(sBaseOSDir);
}

void OSDir::makeOSDir (const char* dirName)
{
	setOSDir(dirName);
	//	mkdir(sBaseOSDir);
}

const char* OSDir::getStdName (void)
{
	return sBaseStdDir;
}

const char* OSDir::getOSName (void)
{
	return sBaseOSDir;
}

bool OSDir::getNextFile (OSFile &oFile, bool bRecursive)
{
	if (iNamePos == -1)
	{
		((FileNameList*)pNameList)->clear();
		windowsAddFileStack(sBaseOSDir,bRecursive);
		linuxAddFileStack(sBaseOSDir,bRecursive);

		iNamePos = 0;
	}

	if (iNamePos >= ((FileNameList*)pNameList)->size())
	{
		iNamePos = -1;
		return false;
	}

	oFile.osName((*((FileNameList*)pNameList))[iNamePos].c_str());

	iNamePos++;
	return true;
}

bool OSDir::windowsAddFileStack (const char *pathName, bool bRecursive)
{
#ifdef WIN32
	struct _finddata_t fileInfo;
	
	long		hFile;
	std::string searchstr;

	std::string FilePath;

	bool	bDone = false;

	searchstr = pathName;
	searchstr += "\\*.*";

	hFile = _findfirst(searchstr.c_str(),&fileInfo);

	if (hFile != -1)
	{
		while (!bDone)
		{
			if ((strcmp(fileInfo.name,".") != 0) && (strcmp(fileInfo.name,"..") != 0))
			{
				FilePath = pathName;
				FilePath += "\\";
				FilePath += fileInfo.name;

				if ((fileInfo.attrib & _A_SUBDIR) && bRecursive)
					windowsAddFileStack(FilePath.c_str(),bRecursive);
				else 
					((FileNameList*)pNameList)->push_back(FilePath);
			}
			if (_findnext(hFile,&fileInfo) == -1)
				bDone = true;
		}
	}
	return true;
#else
	return false;
#endif 
}

bool OSDir::linuxAddFileStack (const char *pathName, bool bRecursive)
{
#ifdef WIN32
	return false;//WIN32
#else
	DIR	*directory;
	dirent	*fileInfo;
	struct stat	statbuf;
	char	searchstr[1024];
	std::string	FilePath;

	strcpy(searchstr, pathName);
	if (searchstr[strlen(searchstr) - 1] != '/')
		strcat(searchstr, "/");
	directory = opendir(searchstr);
	if (!directory)
		return false;
	while ((fileInfo = readdir(directory)))
	{
		if (!((strcmp(fileInfo->d_name, ".") == 0) || (strcmp(fileInfo->d_name, "..") == 0)))
		{
			FilePath = searchstr;
			FilePath += fileInfo->d_name;

			stat(FilePath.c_str(), &statbuf);

			if (S_ISDIR(statbuf.st_mode) && bRecursive)
				LinuxAddFileStack(FilePath.c_str(),bRecursive);
			else
				((FileNameList*)pNameList)->push_back(FilePath);
		}
	}
	closedir(directory);
	return true;
#endif// !Win32
}
